/************************************************************
;                                                           *
;  William M. Spears					    *
;  Navy Center for Applied Research in AI                   *
;  Naval Research Laboratory                                *
;                                                           *
;  This software is the property of the Department of the   *
;  Navy. Permission is hereby granted to copy all or any    *
;  part of this program for free distribution, however      *
;  this header is required on all copies.		    *
;                                                           *
; File: ga.c                                                *
;************************************************************/

#include "header.h"

/* Marker to indicate when a run of the GA is done. */

int done;

/* Essentially your standard generation GA. The "done" flag
   is set here when you are finished. This is based on the
   termination criterion. The convergence function can be used
   to exit, allowing restarts in the function "ga_search". */

void run_ga (int pop_size, int number_of_bits, int points, FILE *onlinefp,
	     FILE *offlinefp, FILE *convergefp, FILE *bestfp, FILE *reevalfp)
{
  int i;
  
  cross_points = points;  /* THIS IS NEW. A pointer to the crossover function */
  if (cross_points)       /* is used to keep track of which type of crossover */
    crossover_function = cross_population;  /* is employed. */
  else
    crossover_function = uniform_cross_population;

  time_keeper();
  fitness(onlinefp, offlinefp, convergefp, bestfp, reevalfp);
  offspring();
  for (i = 0; !((done = termination()) || convergencep()); i++) {
    /*	for (i = 0; !(done = termination()); i++) {*/
    
    report();
    ga_select();
    shuffle();
    swap_pops();
    copy_population();
    mutate();

    (*crossover_function)();  /* Call to the crossover function */
    
    time_keeper();
    fitness(onlinefp, offlinefp, convergefp, bestfp, reevalfp);
    offspring();
  }
  return;
}

/* Initialize GA variables and run GA until done. The "done"
   flag allows the GA to restart as often as needed to find
   the solution. */

void ga_search (int pop_size, int number_of_bits, int points, FILE *logfp, 
		FILE *onlinefp, FILE *offlinefp, FILE *convergefp, FILE *bestfp, 
		FILE *reevalfp)
{
  int i;
  
  done = 0;
  init_ga_variables(pop_size, number_of_bits);
  init_ga_population();
  
  for (i = 1; !done; i++) {
    iteration = i;
    printf("Iteration %d of GA at %d generations\n", i, gen_time);
    run_ga(pop_size, number_of_bits, points, 
	   onlinefp, offlinefp, convergefp, bestfp, reevalfp);
    if (!done) reinit_everything();
  }
  
  fprintf(logfp, "Individual ");
  fshow_best_individual(logfp);
  fprintf(logfp, " with score %f found in %d evaluations and %d iterations.",
	  best, evals, --i);
  fprintf(logfp, "\n");
  fflush(logfp);
  return;
}
